
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gEKGdisplay.h"

// ----------------------------------------------------------------------
//
//  Konstruktor und Destruktor
// 
// ----------------------------------------------------------------------

/* Konstruktor */
gEKGdisplay::gEKGdisplay() : Gtk::DrawingArea(), Values(MinWidth - GapWidth), 
                             Plast(MinWidth-1)
{ 
  mVmax = ( EKG_amp[0] + EKG_amp[1] + EKG_amp[2] ) / 3 * 255;
  // Gre Initalisieren
  size( MinWidth, MinWidth ); 
  for ( int i = 0; i < 3; i++ )
    zeroPos[i] = MinWidth / 12 + i * MinWidth / 6;
  // Farben Reservieren
  Gdk_Colormap cmap( Gdk_Colormap::get_system() );
  BGcolor = cmap.white();
  FGcolor = cmap.black();
  cmap.alloc( BGcolor );
  cmap.alloc( FGcolor );
  // Pipe zur Prozekommunikation
  if ( 0 != pipe( Pipe ) ) 
    throw string("gEKGdisplay::gEKGdisplay() : Can't open pipe.");
  Gtk::Main::input.connect( slot( this, &gEKGdisplay::drawRealtime ), 
                            Pipe[0], GDK_INPUT_READ );
}


/* Destruktor */
gEKGdisplay::~gEKGdisplay() { }


// ----------------------------------------------------------------------
//
// berschriebene Methoden von Widget
// 
// ----------------------------------------------------------------------

/* Erstbenutzung und gre ndern. */
gint gEKGdisplay::configure_event_impl(GdkEventConfigure* p0)
{
  win = get_window();
  win.set_background( BGcolor );
  Values.resize( width() - GapWidth );
  for ( int i = 0; i < 6; i++ )
    zeroPos[i] = height() / 12 + i * height() / 6;
  if ( Plast >= width() ) 
    Plast = width() - 1;
  return true; // Event erfolgreich behandelt
}


/* berschriebene Funktion von Widget. Zeichnet das Display. */
gint gEKGdisplay::expose_event_impl(GdkEventExpose* p0)
{
  gc.create( win );
  gc.set_foreground( FGcolor );
  win.clear_area( p0->area.x, p0->area.y, p0->area.width, p0->area.height );
  gc.set_clip_rectangle( &p0->area );

  gint x1 = addX( Plast, GapWidth + 1 );
  gint x2;
  gint y[12];
  gint * yswap;
  gint * y1 = y; 
  gint * y2 = y + 6;
  scaleY( Values[0], y1 );
  for ( int i = 1; i < Values.size(); i++)
  {
    scaleY( Values[i], y2 );
    x2 = addX( x1, 1 );
    if ( x2 != 0 ) // nicht ber den rechten Rand malen
      for ( int k = 0; k < 6; k++ )
        win.draw_line( gc, x1, y1[k], x2, y2[k] );
    x1 = x2;
    yswap = y1;
    y1 = y2;
    y2 = yswap;
  }
  gc.set_noclip();
  return true; // Event erfolgreich behandelt
}


// ----------------------------------------------------------------------
//
// berschriebene Methoden EKGdevice
// 
// ----------------------------------------------------------------------

/* Daten annehmen, speichern und darstellen */
void gEKGdisplay::input( float * d )
{
  if ( dsize != write( Pipe[1], d, dsize ) )
    throw string("gEKGdisplay::input() : Can't write dsize bytes to pipe.");
  EKGdevice::input( d );
}


// ----------------------------------------------------------------------
//
// Public Methoden
// 
// ----------------------------------------------------------------------

/* Alle Werte Lschen */
void gEKGdisplay::reset()
{
  Values.reset();
  Plast = width();
  draw( Gdk_Rectangle( 0, 0, width(), height() ) );
}

// ----------------------------------------------------------------------
//
// Private Methoden
// 
// ----------------------------------------------------------------------


// ----------------------------------------------------------------------
// Darstellung
// ----------------------------------------------------------------------

/* Echtzeit Darstellung. Wird von Gtk::Main::input event aufgerufen. 
 * Die Daten werden aus der Pipe gelesen. */
void gEKGdisplay::drawRealtime( gint fd, GdkInputCondition /* not used */ )
{
  // Daten von Pipe holen
  float d[3];
  if ( dsize != read( fd, d, dsize ) )
    throw string("gEKGdisplay::drawRealtime() : Can't get dsize bytes.");
  // Zeichnen
  gint y1[6];
  gint y2[6];
  gint x2 = addX( Plast, 1 );
  gc.create( win );
  // zeichnen, es sei denn, wir sind am rechten Rand.
  if ( Plast != width() - 1 )
  {
    gc.set_foreground( FGcolor );
    gc.set_clip_rectangle( Plast, 0, 2, height() );
    scaleY( Values[ Values.size()-1 ], y1 );
    scaleY( d, y2 );
   for ( int i = 0; i < 6; i++ )
      win.draw_line( gc, Plast, y1[i], x2, y2[i] );
  }
  // lteste Werte vom Bildschirm lschen.
  gc.set_foreground( BGcolor );
  gint oldVal = addX( Plast, GapWidth );
  gc.set_clip_rectangle( oldVal, 0, 1, height() );
  win.draw_line( gc, oldVal, 0, oldVal, height() );
  gc.set_noclip();
  // Bildschirmposition aktualisieren, Werte speichern.
  Plast = x2;
  Values.input( d );
}


// ----------------------------------------------------------------------
// Skalierung
// ----------------------------------------------------------------------

// Alle sechs y-Postitonen
void gEKGdisplay::scaleY( float * y, gint * Y )
{
  Y[0] = scaleY( y[0], 0 );
  Y[1] = scaleY( y[1], 1 );
  Y[2] = scaleY( y[2], 2 );
  Y[3] = scaleY( aVR( y ), 3 );
  Y[4] = scaleY( aVL( y ), 4 );
  Y[5] = scaleY( aVF( y ), 5 );
}

