
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gEKGhelp.h"
#include "help_eindhoven.xpm"
#include "help_goldberger.xpm"
#include "keys.string"
#include "about.string"

// ----------------------------------------------------------------------
//
// Konstruktor
// 
// ----------------------------------------------------------------------

gEKGhelp::gEKGhelp() :
    vbox( false, 5 ),
    bbox( GTK_BUTTONBOX_END ),
    OKButton( "OK" ),
    frame1Left( "Eindhoven" ),
    frame1Right( "Goldberger" ),
    pix1Left( help_eindhoven_xpm ),
    pix1Right( help_goldberger_xpm ),
    vbox1Left( false, 0 ),
    vbox1Right( false, 0 ),
    hbox1( false, 5 ),
    label2( "Label 2" ),
    label3( "Label 3" )
{
  // Aussehen allgemein
  set_wmclass( "EKG", "EKG" );
  set_title( "Help - About" );
  set_policy( false, false, true );
  set_border_width( 5 );
  // Aussehen Fenster
  add( vbox );
  vbox.pack_start( notebook );
  vbox.pack_start( bbox );
  bbox.pack_start( OKButton );
  // Ableitungen
  notebook.pages().push_back(
      Gtk::Notebook_Helpers::TabElem( hbox1, "Leads" ) );
  hbox1.set_border_width( 5 );
  hbox1.pack_start( frame1Left );
  hbox1.pack_start( frame1Right );
  frame1Left.add( vbox1Left );
  frame1Right.add( vbox1Right );
  vbox1Left.set_border_width( 4 );
  vbox1Left.pack_start( frame1pLeft, false, false, 0 );
  vbox1Right.set_border_width( 4 );
  vbox1Right.pack_start( frame1pRight, false, false, 0 );
  frame1pLeft.set_shadow_type( GTK_SHADOW_IN );
  frame1pLeft.add( pix1Left );
  frame1pRight.set_shadow_type( GTK_SHADOW_IN );
  frame1pRight.add( pix1Right );
  // Tasten
  string s2;
  writestring( keys_string, s2 );
  label2.set_text( s2 );
  label2.set_justify( GTK_JUSTIFY_LEFT );
  notebook.pages().push_back(
      Gtk::Notebook_Helpers::TabElem( frame2, "Key Bindings" ) );
  frame2.set_shadow_type( GTK_SHADOW_ETCHED_IN );
  frame2.add( label2 );
  // About
  string s3;
  writestring( about_string, s3 );
  label3.set_text( s3 );
  label3.set_justify( GTK_JUSTIFY_LEFT );
  notebook.pages().push_back( 
      Gtk::Notebook_Helpers::TabElem( frame3, "About" ) );
  frame3.set_shadow_type( GTK_SHADOW_ETCHED_IN );
  win3.add_with_viewport( label3 );
  frame3.add( win3 );
  // Alle zeigen, damit die Gre bekannt ist
  pix1Left.realize(); 
  pix1Right.realize();
  OKButton.show();
  bbox.show();
  pix1Left.show();
  pix1Right.show();
  frame1pLeft.show();
  frame1pRight.show();
  vbox1Left.show();
  vbox1Right.show();
  frame1Left.show();
  frame1Right.show();
  hbox1.show();
  label2.show();
  frame2.show();
  label3.show();
  win3.show();
  frame3.show();
  notebook.show();
  vbox.show();
  // Event Handler
  key_press_event.connect( slot( this, & gEKGhelp::keyCallback ) );
  OKButton.clicked.connect( slot( this, & gEKGhelp::buttonCallback ));
}


// ----------------------------------------------------------------------
//
// private Methoden
// 
// ----------------------------------------------------------------------

// Tastendrcke
gint gEKGhelp::keyCallback( GdkEventKey * e )
{
  switch ( e->keyval )
  {
    case 0xff1b : // escape
      OKButton.clicked();
      return true;
  }
  return false;
}

// Daten in einen String schreiben
void gEKGhelp::writestring( const char ** const s, string & d )
{
  // beim ersten leeren cstring aufhren
  for ( int i = 0; s[i][0]; i++ ) 
    d += s[i];
}

